/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.api;

/**
 * Repesents an extension of TupleStructure where the Fields can be added
 * or removed.
 * It is used by QueryNodes
 * @see org.objectweb.medor.query.api.QueryNode
 * @author	<A HREF="mailto:alia.mourad@rd.francetelecom.com><b>
 *			Mourad Alia
 *			</b></A>
 *		<A HREF="mailto:alexandre.lefebvre@rd.francetelecom.com><b>
 *			Alexandre Lefebvre
 * 			</b></A>
 */

public interface RWTupleStructure {
	/**
	 * Adds a Field to the current RWTupleStructure
	 * @param f The Field to be added.
	 */
	void addField(Field f);

	/**
	 * Removes a Field from the current RWTupleStructure.
	 * @param f The Field to be removed.
	 * @return false if the Field could not be removed (was not part of the
	 * TupleStructure).
	 */
	boolean removeField(Field f);
}
