/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

/**
 * @author	<A HREF="mailto:alia.mourad@rd.francetelecom.com><b>
 *			Mourad Alia
 *			</b></A>
 */

package org.objectweb.medor.datasource.lib;

import org.objectweb.medor.api.DataSourceException;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.api.Wrapper;
import org.objectweb.medor.datasource.api.WrapperFactory;
import org.objectweb.medor.datasource.rdb.lib.JDBCWrapper;

import java.util.HashMap;

/**
 * A simple implementation of the WrapperFactory interface...
 * Wrappers are used to fetch data, so the same wrapper type can be used
 * many times.
 */
public class MedorWrapperFactory implements WrapperFactory {

    public Wrapper getWrapper(DataStore dl) throws DataSourceException{
        Wrapper w;
        if (wrapperPool.containsKey(new Short(dl.getDataStoreType()))) {
            return (Wrapper) wrapperPool.get(new Short(dl.getDataStoreType()));
        } else {
            switch (dl.getDataStoreType()) {
                case (DataStore.MEDORTC_STORE):
                    w = new TCWrapper();
                    wrapperPool.put(new Short(dl.getDataStoreType()), w);
                    return w;
                case (DataStore.JDBC_STORE):
                    w = new JDBCWrapper();
                    wrapperPool.put(new Short(dl.getDataStoreType()), w);
                    return w;
                default :
                    throw new DataSourceException("I don't no wrapping this dataStore type ");
            }
        }
    }

    HashMap wrapperPool = new HashMap();
}
