/**
 * copyright area
 */
package org.objectweb.medor.filter.jorm.lib;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.api.EvaluationException;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.filter.api.CalculatedParameterOperand;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.lib.BasicParameterOperand;
import org.objectweb.medor.type.lib.QType;

/**
 * This implementation of the CalculatedParameterOperand interface encode a
 * a PName. This PName must be given in parameter among the ParameterOperand
 * of the evaluate method.
 * There are two constructor weither the PName structure
 *
 * @author S.Chassande-Barrioz
 */
public class EncodePName
	extends BasicParameterOperand
	implements CalculatedParameterOperand {

	String fieldName = null;

	public EncodePName(PType type, String parameterName) {
		super(type, parameterName);
		fieldName = null;
	}

	public EncodePName(PType type, String parameterName, String fieldName) {
		super(type, parameterName);
		this.fieldName = fieldName;
	}

	// IMPLEMENTATION OF THE CalculatedParameterOperand INTERFACE //
	//------------------------------------------------------------//

	public void evaluate(ParameterOperand[] pos) throws EvaluationException {
		PName pn = null;
		for (int i = 0; i < pos.length; i++) {
			if (name.equals(pos[i].getName())
				&& pos[i].getType().getTypeCode() == QType.TYPECODE_PNAME) {
				pn = (PName) pos[i].getObject();
			}
		}
		if (pn == null)
			throw new EvaluationException("Impossible to evaluate this parameter wihtout the parameter " + name);
		if (pn.isNull())
			throw new EvaluationException("Impossible to evaluate this parameter with a null PName:" + pn);
		try {
			if (fieldName == null) {
				// Single PName
				switch (type.getTypeCode()) {
				case QType.TYPECODE_BYTE:
					setValue(pn.encodeByte());
					break;
				case QType.TYPECODE_CHAR:
					setValue(pn.encodeChar());
					break;
				case QType.TYPECODE_INT:
					setValue(pn.encodeInt());
					break;
				case QType.TYPECODE_LONG:
					setValue(pn.encodeLong());
					break;
				case QType.TYPECODE_SHORT:
					setValue(pn.encodeShort());
					break;
				case QType.TYPECODE_OBJBYTE:
					setValue(pn.encodeObyte());
					break;
				case QType.TYPECODE_OBJCHAR:
					setValue(pn.encodeOchar());
					break;
				case QType.TYPECODE_OBJINT:
					setValue(pn.encodeOint());
					break;
				case QType.TYPECODE_OBJLONG:
					setValue(pn.encodeOlong());
					break;
				case QType.TYPECODE_OBJSHORT:
					setValue(pn.encodeOshort());
					break;
				case QType.TYPECODE_DATE:
					setValue(pn.encodeDate());
					break;
				case QType.TYPECODE_STRING:
					setValue(pn.encodeString());
					break;
				default:
					throw new EvaluationException("Unsupported naming type" + type);
				}
			}
			else {
				// Composite PName
				Object conn = null;
				PNameGetter png = (PNameGetter) pn.encodeAbstract();
				switch (type.getTypeCode()) {
				case QType.TYPECODE_BYTE:
					setValue(png.pngetByteField(fieldName, conn));
					break;
				case QType.TYPECODE_CHAR:
					setValue(png.pngetCharField(fieldName, conn));
					break;
				case QType.TYPECODE_INT:
					setValue(png.pngetIntField(fieldName, conn));
					break;
				case QType.TYPECODE_LONG:
					setValue(png.pngetLongField(fieldName, conn));
					break;
				case QType.TYPECODE_SHORT:
					setValue(png.pngetShortField(fieldName, conn));
					break;
				case QType.TYPECODE_OBJBYTE:
					setValue(png.pngetObyteField(fieldName, conn));
					break;
				case QType.TYPECODE_OBJCHAR:
					setValue(png.pngetOcharField(fieldName, conn));
					break;
				case QType.TYPECODE_OBJINT:
					setValue(png.pngetOintField(fieldName, conn));
					break;
				case QType.TYPECODE_OBJLONG:
					setValue(png.pngetOlongField(fieldName, conn));
					break;
				case QType.TYPECODE_OBJSHORT:
					setValue(png.pngetOshortField(fieldName, conn));
					break;
				case QType.TYPECODE_DATE:
					setValue(png.pngetDateField(fieldName, conn));
					break;
				case QType.TYPECODE_STRING:
					setValue(png.pngetStringField(fieldName, conn));
					break;
				default:
					throw new EvaluationException("Unsupported naming type" + type);
				}
			}
		}
		catch (TypingException e) {
			throw new EvaluationException("Impssible to assign parameter", e);
		}
		catch (PException e) {
			throw new EvaluationException("Impssible to encode the PName", e);
		}
	}
}
