/**
 * copyright area
 */
package org.objectweb.medor.filter.lib;

import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.api.Operand;
import org.objectweb.medor.filter.api.CalculatedParameterOperand;
import org.objectweb.medor.filter.api.BinaryOperator;
import org.objectweb.medor.filter.api.UnaryOperator;
import org.objectweb.medor.filter.api.Operator;
import org.objectweb.medor.filter.jorm.lib.CompositePName;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.api.Field;
import org.objectweb.jorm.type.api.PType;

/**
 * @author S.Chassande-Barrioz
 */
public class ExpressionPrinter {
	public static String e2str(Expression e) {
		if (e instanceof FieldOperand) {
			Field f = ((FieldOperand) e).getField();
			if (f == null)
				return ((FieldOperand) e).getType().getJavaName() + "nullFO";
			else
				return f.getName();
		}
		else if (e instanceof CompositePName) {
			FieldOperand[] fields = ((CompositePName) e).getFields();
			String result = "CompositePName(";
			Field f;
			for(int i=0; i<fields.length; i++) {
				f = fields[i].getField();
				if (f==null)
					result += "no field ,";
				else
					result += f.getName() + ", ";
			}
            result += ((CompositePName) e).getExpression(0);
			result += ")";
            return result;
		}
		else if(e instanceof SinglePName) {
            String name = ((ParameterOperand) ((SinglePName) e)
                .getRightExpression()).getName();
			Field f = ((SinglePName) e).getField().getField();
			if (f == null)
				return "no field, paramName=" + name;
			else
				return "SinglePName(" + f.getName() + ", paramName=" + name + ")";
		}
		else if (e instanceof Operand) {
			PType type = ((Operand) e).getType();
			if (e instanceof ParameterOperand) {
				return
					(e instanceof CalculatedParameterOperand
						? "CalculatedParam("
						: "Param(")
					+ ((ParameterOperand) e).getName() + ", "
					+ (type == null ? "nullType" : type.getJormName()) + ","
					+ e.toString() + ")";
			}
			else
				return e.toString();
		} else if (e instanceof BinaryOperator) {
			BinaryOperator bo = (BinaryOperator) e;
			return "("
				+ e2str(bo.getLeftExpression())
				+ " " + bo.getOperatorString() + " "
				+ e2str(bo.getRightExpression())
				+ ")";
		} else if (e instanceof UnaryOperator) {
			UnaryOperator uo = (UnaryOperator) e;
			return uo.getOperatorString()
				+ e2str(uo.getExpression());
		} else if (e instanceof Operator) {
			Operator op = (Operator) e;
			StringBuffer s = new StringBuffer();
			s.append(op.getOperatorString());
			for (int i=0; i< op.getOperandNumber(); i++) {
				s.append((i==0 ? "(" : ","));
				s.append(op.getExpression(i));
			}
			s.append(")");
			return s.toString();
		}
		return "";
	}

}
