/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.filter.lib;

import org.objectweb.medor.filter.api.BinaryOperator;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.Operator;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TypingException;
import org.objectweb.medor.api.MalformedExpressionException;
import org.objectweb.medor.type.lib.QTypeTuple;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.type.lib.QType;
import org.objectweb.jorm.type.api.PTypeSpace;

import java.sql.Timestamp;
import java.sql.Time;
import java.util.Date;

public class FirstLocate extends BasicBinaryOperator implements BinaryOperator {

    public FirstLocate(Expression left, Expression right) {
        this.left = left;
        this.right = right;
    }

    public FirstLocate() {
    }

    public void evaluate(ParameterOperand[] pos, Tuple t)
            throws MedorException, IllegalStateException {
        try {
            left.evaluate(pos, t);
            right.evaluate(pos, t);
            result.setValue(evaluate(leftResult.getString(), rightResult.getString()));
        } catch (NullPointerException e) {
            // result == null or missing children objects
            throw new IllegalStateException("Unevaluable Expression: Not compiled");
        }
    }

    public int evaluate(String op1, String op2) {
        return op1.indexOf(op2);
    }

    public void compileExpression()
            throws TypingException, MalformedExpressionException {
        if ((left != null) && (right != null)) {
            left.compileExpression();
            leftResult = left.getResult();
            right.compileExpression();
            rightResult = right.getResult();
            if (!(left.getType() == null || right.getType() == null)) {
                if ((left.getType().getTypeCode() == QType.TYPECODE_STRING) &&
                        (right.getType().getTypeCode() == QType.TYPECODE_STRING)) {
                    type = PTypeSpaceMedor.INT;
                    result = new BasicVariableOperand(type);
                    verified = true;
                } else
                // it is not a string type
                    throw new TypingException("Attempt arg type : String, String");
            } else
                throw new MalformedExpressionException("null children value");
        } else
        // Operands not yet assigned
            throw new MalformedExpressionException("null children value");
    }

    public String getOperatorString() {
        return Operator.FIRSTLOCATE;
    }
}
