/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, S. Chassande-Barrioz, A. Lefebvre
 */

package org.objectweb.medor.optim.lib;

import org.objectweb.medor.api.EvaluationException;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.ExpressionHelper;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.lib.And;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.postfix.PostfixExpressionHelper;
import org.objectweb.medor.optim.api.RewriteRule;
import org.objectweb.medor.query.api.CalculatedField;
import org.objectweb.medor.query.api.OperationType;
import org.objectweb.medor.query.api.PropagatedField;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.rdb.api.QualifiedTable;
import org.objectweb.medor.query.rdb.api.RdbExpField;
import org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf;
import org.objectweb.medor.query.rdb.lib.BasicRdbExpField;
import org.objectweb.medor.query.rdb.lib.BasicRdbExpQueryLeaf;

import java.util.HashMap;
import java.util.Stack;
import java.util.Vector;

/**
 * This class represents the rule to group as a single QueryLeaf the QueryNodes
 * and QueryLeaves which apply to the same data store (for example into an
 * RdbQueryLeaf).
 */
public class GroupSameStoreRule extends BasicRule {

	HashMap changedField = new HashMap();
	Stack queryTreeStack = new Stack();
	ExpressionHelper eh = new PostfixExpressionHelper();

	public GroupSameStoreRule() {
		super("GroupSameStoreRule");
	}

	public QueryTree rewrite(QueryTree qt) throws MedorException {
		toStack(qt);
		int i,cpt;
		int nbreQL;
		DataStore rdbDS;
		RdbExpQueryLeaf newRdbExpQL = null;
		for (cpt = 0; (cpt < queryTreeStack.size()); cpt++) {
			if (queryTreeStack.elementAt(cpt) instanceof QueryNode) {
				nbreQL = ((QueryNode) queryTreeStack.elementAt(cpt)).getChildren().length;
				if (!isThereCalculatedField(((QueryNode) queryTreeStack.elementAt(cpt)).getTupleStructure())) {
					QueryTree[] children =
						((QueryNode) queryTreeStack.elementAt(cpt)).getChildren();
					RdbExpQueryLeaf[] rdbChild = new RdbExpQueryLeaf[nbreQL];
					if (children[0] instanceof RdbExpQueryLeaf) {
						rdbDS = ((RdbExpQueryLeaf) children[0]).getDataStore();
						rdbChild[0] = (RdbExpQueryLeaf) children[0];
						// Tests if all QueryLeaf children of the QueryNode have the same Rdb DataStore
						for (i = 1;
							 (i < nbreQL &&
							(children[i] instanceof RdbExpQueryLeaf &
							((QueryLeaf) children[i]).getDataStore().isSameAs(rdbDS)));
							 i++) {
							rdbChild[i] = (RdbExpQueryLeaf) children[i];
						}
						if (i == nbreQL) {
							// Construct the ExpQueryLeaf wich groups the QueryNode and its RdbExpQueryLeaf children
							RdbExpQueryLeaf groupedExpQL =
								merge((QueryNode) queryTreeStack.elementAt(cpt),
									rdbChild);
							// Updates all depandences and links
							updatesLinks(cpt, groupedExpQL);
							// Gets the head of the rewrited QueryTree
							newRdbExpQL = groupedExpQL;
						}
						else
							newRdbExpQL =
								(RdbExpQueryLeaf) queryTreeStack.elementAt(cpt);
					}
					else
						newRdbExpQL =
							(RdbExpQueryLeaf) queryTreeStack.elementAt(cpt);
				}
				else
					newRdbExpQL =
						(RdbExpQueryLeaf) queryTreeStack.elementAt(cpt);
			}
		}
		return newRdbExpQL;
	}

	protected RdbExpQueryLeaf merge(QueryNode medorOperation,
									RdbExpQueryLeaf[] rdbExpQueryLeaf)
		throws MedorException {
		// We get the DataStore of the first QueryLeaf
		DataStore ds = rdbExpQueryLeaf[0].getDataStore();
		// We get the First RdbExpExpQueryLeaf name
		String name = rdbExpQueryLeaf[0].getName();
		// We get the QualifiedTables
		HashMap tables = new HashMap();
		QualifiedTable[] qtables;
		Vector allQtables = new Vector();
		int aliasNum = 1;// It contains the current number of alias
		for (int i = 0; (i < rdbExpQueryLeaf.length); i++) {
			qtables = rdbExpQueryLeaf[i].getQualifiedTables();
			for (int cpt = 0; (cpt < qtables.length); cpt++) {
				/*
                 * Note that a table may have many alias...
                 * If same alias for the same table in two RdbExpQueryLeaf
                 * then we must rename alias
                 */
				if (tables.containsKey(qtables[cpt].getTableName())) {
					//if (((QualifiedTable) (tables.get(qtables[cpt].getTableName()))).getAliasName() != null) {
					if (!(((Vector) (tables.get(qtables[cpt].getTableName()))).contains(qtables[cpt].getAliasName()))) {
						// We add this alias to the list of the current table's alias
						((Vector) (tables.get(qtables[cpt].getTableName()))).add(qtables[cpt]);
						allQtables.add(qtables[cpt]);
					}
					else { // The same alias...so renaming
						if (qtables[cpt].getAliasName() == null)
							qtables[cpt].setAliasName(qtables[cpt].getTableName() + "_alias");
						else
							qtables[cpt].setAliasName(qtables[cpt].getAliasName() + aliasNum++);
						// To verify if the name is diffent from other names
						cpt--;
					}
				}
				else { // Creating a new Collection for alias
					Vector list = new Vector();
					list.add(qtables[cpt].getAliasName());
					tables.put(qtables[cpt].getTableName(), list);
					allQtables.add(qtables[cpt]);
				}
			}
		}
		// We have the same QualifiedTable Object, we group them into a QualifiedTable[]

		RdbExpQueryLeaf expResultLeaf = new BasicRdbExpQueryLeaf(ds, (QualifiedTable[]) allQtables.toArray(new QualifiedTable[0]), null);
		if ((medorOperation.getType() == OperationType.PROJECT) ||
			(medorOperation.getType() == OperationType.SELECT_PROJECT) ||
			(medorOperation.getType() == OperationType.JOIN_PROJECT)) {
			rewriteTupleStructure(medorOperation.getTupleStructure(), expResultLeaf);
			Expression exp = rewriteQueryNodeFilter(medorOperation.getQueryFilter());
			for (int i = 0; (i < rdbExpQueryLeaf.length); i++) {
				exp = mergeExpressions(exp, rdbExpQueryLeaf[i].getQueryFilter());
			}
			expResultLeaf.setQueryFilter(exp);
		}
		else
			switch (medorOperation.getType()) {
			case (OperationType.UNION):
			case (OperationType.INTERSECTION):
			case (OperationType.CARTESIAN):
			case (OperationType.NEST):
			}
		return expResultLeaf;
	}

	protected Expression mergeExpressions(Expression leftQLExp,
										  Expression rightQLExp) throws MedorException {
		// Construct The new merged QueryFilter
		if ((leftQLExp == null) && (rightQLExp != null))
			return rewriteQueryLeafFilter(rightQLExp);
		else if ((leftQLExp != null) && (rightQLExp == null))
			return rewriteQueryLeafFilter(leftQLExp);
		else if ((leftQLExp == null) && (rightQLExp == null))
			return null;
		else
			return new And(rewriteQueryLeafFilter(leftQLExp),
				rewriteQueryLeafFilter(rightQLExp));
	}

	private Expression rewriteQueryNodeFilter(Expression fatherExp) throws MedorException {
		Expression[] stack = eh.toStack(fatherExp);
		for (int cpt = 0; (cpt < stack.length); cpt++) {
			if (stack[cpt] instanceof FieldOperand) {
				FieldOperand fop = (FieldOperand) stack[cpt];
				RdbExpField pField = (RdbExpField) fop.getField();
				FieldOperand newFop;
				RdbExpField nfield = (RdbExpField) changedField.get(pField);
				if (nfield != null) {
					newFop = new BasicFieldOperand(nfield);
				}
				else {
					Field f = new BasicRdbExpField(pField.getName(),
						pField.getType(), pField.getColumnName(), pField.getTable(), null);
					newFop = new BasicFieldOperand(f);
				}
				stack[cpt] = newFop;
			}
		}
		return fatherExp;
	}

	private Expression rewriteQueryLeafFilter(Expression e) throws MedorException {
		Expression[] stack = eh.toStack(e);
		for (int cpt = 0; (cpt < stack.length); cpt++) {
			if (stack[cpt] instanceof FieldOperand) {
				FieldOperand fop = (FieldOperand) stack[cpt];
				RdbExpField rdbExpField = (RdbExpField) fop.getField();
				FieldOperand newFop;
				RdbExpField nfield = (RdbExpField) changedField.get(rdbExpField);
				if (nfield != null) {
					newFop = new BasicFieldOperand(nfield);
				}
				else {
					Field f = new BasicRdbExpField(rdbExpField.getName(),
						rdbExpField.getType(), rdbExpField.getColumnName(),
						rdbExpField.getTable(), null);
					newFop = new BasicFieldOperand(f);
				}
				stack[cpt] = newFop;
			}
		}
		return e;
	}

	protected void rewriteTupleStructure(TupleStructure ts,
										 RdbExpQueryLeaf expResultLeaf) throws MedorException {
		Field[] fields = ts.getFields();
		for (int cpt = 0; (cpt < fields.length); cpt++) {
			if (fields[cpt] instanceof PropagatedField) {
				Field[] previous = ((PropagatedField) fields[cpt]).getPreviousFields();
				// We take the first previous child
				changedField.put(fields[cpt], expResultLeaf.addRdbField(previous[0].getName(),
					previous[0].getType(), ((RdbExpField) previous[0]).getColumnName(),
					((RdbExpField) previous[0]).getTable()));
			}
			else if (fields[cpt] instanceof CalculatedField) {
				Expression exp = this.rewriteQueryNodeFilter(((CalculatedField) fields[cpt]).getExpression());
				// TO BE CONTININUED
				changedField.put(fields[cpt], fields[cpt]);
			}
		}
	}

	private boolean isThereCalculatedField(TupleStructure ts) {
		Field[] fields = ts.getFields();
		int cpt;
		for (cpt = 0; (cpt < fields.length && !(fields[cpt] instanceof CalculatedField)); cpt++) ;
		if (cpt == fields.length)
			return false;
		else
			return true;
	}

	private void toStack(QueryTree query) throws EvaluationException {
		toStack(query, queryTreeStack);
	}

	private void toStack(QueryTree query, Stack pile) throws EvaluationException {
		if (query instanceof QueryLeaf) {
			pile.push(query);
		}
		else {
			QueryTree[] children = ((QueryNode) query).getChildren();
			for (int cpt = 0; (cpt < children.length); cpt++) {
				toStack(children[cpt], pile);
			}
			pile.push(query);
		}
	}

	private void updatesLinks(int cpt, RdbExpQueryLeaf rdbExpQL) throws MedorException {
		boolean stop = false;
		boolean theFirst = true;
		// Retriving the father of the QueryNode so rank = cptin the stack
		int i = cpt + 1;
		// Test if we are at the head
		if (i < queryTreeStack.size()) {
			while (!stop && i < queryTreeStack.size()) {
				if (queryTreeStack.elementAt(i) instanceof QueryLeaf) {
					theFirst = false;
				}
				else if (theFirst)
					stop = true;
				else
					theFirst = true;
			}
			// Updating links now
			QueryNode qnFather = (QueryNode) queryTreeStack.elementAt(i);
			QueryNode qnChild = (QueryNode) queryTreeStack.elementAt(cpt);
			// Updating FieldOperand links
			Expression[] sExp = eh.toStack(qnFather.getQueryFilter());
			for (int ii = 0; (ii < sExp.length); ii++) {
				PropagatedField pf = null;
				if (sExp[ii] instanceof FieldOperand)
					pf = (PropagatedField) ((FieldOperand) sExp[ii]).getField();
				Field newField = (Field) changedField.get(pf);
				if (newField != null) {// It's one of the Child's Field
					pf.replacePreviousField(new Field[]{(Field) changedField.get(pf)});
					((FieldOperand) sExp[ii]).setField(newField);
				}
			}
			// Updating FieldOperand Links
			PropagatedField[] pfields = (PropagatedField[]) qnFather.getTupleStructure().getFields();
			for (int ii = 0; (ii < pfields.length); ii++) {
				if (changedField.get(pfields[ii]) != null) {
					pfields[ii].replacePreviousField(new Field[]{(Field) changedField.get(pfields[ii])});
				}
			}
		}
	}
}