/**
 * copyright area
 */
package org.objectweb.medor.query.jorm.lib;

import org.objectweb.medor.query.jorm.api.JormExtent;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.Field;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.metainfo.api.Reference;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * This class is the common implementation of the jorm leaves (ClassExtent,
 * GenClassExtent).
 *
 * @author S.Chassande-Barrioz
 */
public abstract class BasicJormExtent
	implements JormExtent, QueryLeaf, TupleStructure {
    /**
     * Nond name
     */
    protected String name;

    /**
     * nam of the field which is the PName of the extent
     */
    protected String pnFieldName;

    /**
     * List of fields
     */
	protected ArrayList fields = new ArrayList();

    /**
     * List of fields indexed by their name
     */
    protected HashMap name2field = new HashMap();

    /**
     * The datastore of the extent
     */
    protected DataStore myStore;

    /**
     * The mapper of the extent.
     */
    protected PMapper mapper;

    public BasicJormExtent(String _name, String pnFieldName) {
        this.name = (_name==null ? "" : _name);;
        this.pnFieldName = pnFieldName;
    }

    public Iterator iterateFields() {
		return fields.iterator();
	}

    public String getFieldName(String nodeName, String fieldName) {
        if (nodeName==null || nodeName.length()==0)
            return fieldName;
        else
            return nodeName + "." + fieldName;
    }

    // IMPLEMENTATION OF THE JormExtent INTERFACE //
    //--------------------------------------------//

    public PMapper getPMapper() {
		return mapper;
	}

	public void setPMapper(PMapper m) {
		mapper = m;
	}

    public void setDataStore(DataStore ds) {
        myStore = ds;
    }

    public String getPNameFieldName() {
        return getFieldName(name, pnFieldName);
    }

    // IMPLEMENTATION OF THE QueryLeaf INTERFACE //
    //-------------------------------------------//

    public String getName() {
        return this.name;
    }

	/**
	 * Methods inherited from QueryTree
	 */
	public TupleStructure getTupleStructure() {
		return (TupleStructure) this;
	}

	/**
	 * Methods inherited from QueryLeaf
	 */
	public DataStore getDataStore() {
		return myStore;
	}

	public Object clone() {
//TODO
		return this;
	}

    // IMPLEMENTATION OF THE TupleStructure INTERFACE //
    //------------------------------------------------//

	public Field[] getFields() {
		 return (Field[]) fields.toArray(new Field[0]);
	 }

	 public Field getField(String fieldname) throws MedorException {
		 Field res = (Field) name2field.get(fieldname);
		 if (res==null) {
                throw new MedorException("Field name error: " + fieldname);
		 }
		 return res;
	 }

	 public Field getField(int fieldrank) throws MedorException {
		 Field f = (Field) fields.get(fieldrank - 1);
		 if (f == null) {
			 throw new MedorException("Field rank error : " + fieldrank);
		 } else {
			 return f;
		 }
	 }

	 public int getFieldRank(Field f) throws MedorException {
		 int idx = fields.indexOf(f);
		 if (idx == -1)
			 throw new MedorException("Field not found: " + f.getName());
		 else return idx+1;
	 }


	 public int getSize() {
		 return fields.size();
	 }

	public boolean contains(Field f) {
		return fields.contains(f);
	}

    public boolean contains(String fieldName) {
        return name2field.containsKey(fieldName);
    }
}
