/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.jorm.lib;

import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.TupleStructure;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.query.api.QueryLeaf;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * This class represents the extent of a JORM GenClass.
 * It contains and implements its associated TupleStructure.
 */
public class GenClassExtent extends BasicJormExtent {

	private GenClassRef myClass;
    private String elementFieldName;

	/**
	 * Constructs an extent for the JORM GenClass, in the form of a QueryLeaf.
	 * @param jormClass is the JORM full class name.
	 * @param _name is the name of the node (null value => empty string)
	 * @param classPNameOnly indicates whether the extent should contain only
	 * the PName for the class (if true) or all attributes for that class
	 * (if false).
	 *
	 */
	public GenClassExtent(GenClassRef gcr,
						  String _name,
						  String pnameFieldName) throws MedorException {
        super(_name, pnameFieldName);
		myClass = gcr;

		Field f;
		//Add the Genclass PName
		f = new PNameField(getFieldName(name, pnFieldName), gcr, true, this);
		fields.add(f);
		name2field.put(f.getName(), f);

		// Add the element
        elementFieldName = getFieldName(name, getGCRElemFieldName());
		if (gcr.isClassRef() || gcr.isGenClassRef())
			f = new PNameField(elementFieldName, gcr, false, this);
		else if (gcr.isPrimitive()) {
			f = new BasicJormField(elementFieldName, this, gcr.getPrimitiveElement());
		}
		fields.add(f);
		name2field.put(f.getName(), f);

		// Add the indexes
		for (Iterator idxIt = gcr.getGenClass().iterateIndexField();
			 idxIt.hasNext();) {
			TypedElement te1 = (TypedElement) idxIt.next();
			f = new BasicJormField(
				getFieldName(name, te1.getName()), this, gcr);
			fields.add(f);
			name2field.put(f.getName(), f);
		}
	}

    public String getElementFieldName() {
        return elementFieldName;
    }

	public GenClassRef getMetaObject() {
		return myClass;
	}

	public static String getGCRElemFieldName() {
		return "element";
	}
}
