/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.jorm;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.comp.compiler.lib.BasicCompiler;
import org.objectweb.jorm.comp.compiler.api.CompilerParameter;
import org.objectweb.jorm.metainfo.lib.BasicManager;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.util.io.lib.DirJavaExplorer;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.medor.lib.TestMedorHelper;
import org.objectweb.medor.optim.jorm.JormLeafRewriter;
import org.objectweb.medor.query.jorm.lib.ClassExtent;
import org.objectweb.medor.query.TestQueryTreeHelper;
import org.objectweb.medor.query.lib.QueryTreePrinter;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.api.MedorException;
import org.apache.tools.ant.types.DTDLocation;

import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;

import junit.framework.TestCase;

/**
 * @author S.Chassande-Barrioz
 */
public abstract class TestJormHelper extends TestCase {

    public static final String DEFAULT_PNAME_FIELD_NAME = "object";

    public Manager manager = null;
    public PMapper mapper = null;
    public Logger logger = null;

    public class Example {
        public QueryTree qt;
        public Class clazz;
        public ClassExtent extent;
        public String pnameFieldName;
    }

    public TestJormHelper() {
        this("TestJormHelper");
    }

    public TestJormHelper(String n) {
        super(n);
        manager = new BasicManager();
        mapper = newMappper();
        mapper.setMapperName(getMapperName());
        mapper.setMappingName(getMappingName());
        mapper.setMetaInfoManager(manager);
        logger = new TestMedorHelper();
    }

    public abstract String getPDPath();
    public abstract String getDTDPath();

    public abstract PMapper newMappper();

    public abstract String getMapperName();

    public abstract String getMappingName();

    public abstract String getMIMappingFactory();

    public abstract String getMappingParser();

    public abstract JormLeafRewriter createJormLeafRewriter();

    protected void setUp() {
    }

    protected void tearDown() {
    }

    public Example getExtent(Collection fileNames, String className) {
        return getExtent(fileNames, className, DEFAULT_PNAME_FIELD_NAME, null);
    }

    public Example getExtent(Collection fileNames,
                             String className,
                             String pnameFieldName,
                             String[] fieldNames) {
        Collection mos = null;
        logger.log(BasicLevel.DEBUG, "getExtentOf" + className);
        try {

            mos = loadJormMetaInfo(fileNames);
        }
        catch (PException e) {
            e.printStackTrace(System.out);
            fail(e.getMessage());
        }
        logger.log(BasicLevel.DEBUG, "Jorm meta information loaded");
        for (Iterator it = mos.iterator(); it.hasNext();) {
            MetaObject mo = (MetaObject) it.next();
            if (mo instanceof Class
                && className.equals(((Class) mo).getName())) {
                Example e = new Example();
                e.pnameFieldName = pnameFieldName;
                e.clazz = (Class) mo;
                try {
                    if (fieldNames == null)
                        e.extent = new ClassExtent(
                            e.clazz, e.clazz.getFQName(), pnameFieldName, false);
                    else
                        e.extent = new ClassExtent(
                            e.clazz, e.clazz.getFQName(), fieldNames, true, pnameFieldName);
                    logger.log(BasicLevel.DEBUG, "Extent created for the class "
                        + e.clazz.getName());
                    e.extent.setPMapper(mapper);

                    if (TestMedorHelper.debug)
                        QueryTreePrinter.printQueryTree(e.extent, logger);
                    return e;
                }
                catch (MedorException me) {
                    me.printStackTrace();
                    fail(me.getMessage());
                }
            }
        }
        return null;
    }

    public Example getRewrited(Example e) {
        if (e==null)
            return null;
        try {
            JormLeafRewriter jlr = createJormLeafRewriter();
            logger.log(BasicLevel.DEBUG, "JormLeafRewriter created");

            assertTrue("Unable to rewrite " + e.clazz.getName(), jlr.canRewrite(e.extent));

            e.qt = jlr.rewrite(e.extent);
            logger.log(BasicLevel.DEBUG, "Extent of the class "
                + e.clazz.getName() + " rewrited");

            if (TestMedorHelper.debug)
                QueryTreePrinter.printQueryTree(e.qt, logger);
        }
        catch (MedorException me) {
            me.printStackTrace();
            fail(me.getMessage());
        }
        return e;
    }

    public Collection loadJormMetaInfo(String fileName) throws PException {
        ArrayList al = new ArrayList(1);
        al.add(fileName);
        return loadJormMetaInfo(al);
    }

    public Collection loadJormMetaInfo(Collection fileNames) throws PException {
        BasicCompiler compiler = new BasicCompiler();
        CompilerParameter cp = compiler.getCompilerParameter();
        DirJavaExplorer exp = new DirJavaExplorer();
        exp.addPath(getPDPath());
        cp.setClasspath(exp);
        cp.setMapperName(getMapperName());
        cp.setMappingName(getMappingName());
        cp.setMappingFactory(getMIMappingFactory());
        cp.setMetaInfoManager(manager);
        cp.setParserName(getMappingParser());
        ArrayList al = new ArrayList(1);
        DTDLocation dtdl = new DTDLocation();
        dtdl.setPublicId("-//ObjectWeb Consortium//DTD JORM DEFINITIONS 1.0//EN");
        dtdl.setLocation(getDTDPath());
        al.add(dtdl);
        cp.setDtdLocations(al);
        Logger l = new TestMedorHelper();
        l.turnOff();
        cp.setLogger(l);
        cp.setOutputDirectory(getPDPath());
        return compiler.parseFiles(fileNames);
    }

    public String getFieldName(String alias, String fn) {
        return (alias==null || alias.length()==0 ? "" : alias + ".") + fn;
    }
}
