/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.lib;

import junit.framework.Assert;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.Level;
import org.objectweb.util.monolog.api.BasicLevel;

import java.io.PrintStream;

/*
 * @author S.Chassande-Barrioz
 */

public class TestMedorHelper extends Assert implements Logger {

	public static boolean debug = false;
	public static PrintStream out = System.out;

	static {
		debug = System.getProperty("medor.test.debug", "false").equalsIgnoreCase("true");
	}

	public static void debug(String m) {
		if (debug) System.out.println(m);
	}

	public boolean isOn;

    public TestMedorHelper() {
        isOn = debug;
    }

    public void setIntLevel(int i) {}
	public void setLevel(Level level) {}
	public int getCurrentIntLevel() { return BasicLevel.DEBUG; }

	public Level getCurrentLevel() { return BasicLevel.LEVEL_DEBUG; }
	public boolean isLoggable(int i) { return true; }
	public boolean isLoggable(Level level) {return true; }
	public boolean isOn() { return true;}

	public void log(int i, Object o) {
		if (isOn) out.println(o);
	}

	public void log(Level level, Object o) {
		if (isOn) out.println(o);
	}

	public void log(int i, Object o, Throwable throwable) {
		if (isOn) {
			throwable.printStackTrace(out);
			out.println(o);
		}
	}

	public void log(Level level, Object o, Throwable throwable) {
		if (isOn) {
			throwable.printStackTrace(out);
			out.println(o);
		}
	}

	public void log(int i, Object o, Object o1, Object o2) {
		if (isOn) out.println(o1 + " ." + o2 + ": " + o);
	}

	public void log(Level level, Object o, Object o1, Object o2) {
		if (isOn) out.println(o1 + " ." + o2 + ": " + o);
	}

	public void log(int i, Object o, Throwable throwable, Object o1, Object o2) {
		if (isOn) {
			throwable.printStackTrace(out);
			out.println(o1 + " ." + o2 + ": " + o);
		}
	}

	public void log(Level level, Object o, Throwable throwable, Object o1, Object o2) {
		if (isOn) {
			throwable.printStackTrace(out);
			out.println(o1 + " ." + o2 + ": " + o);
		}
	}
	public void turnOn() {
        isOn = true;
	}
	public void turnOff() {
        isOn = false;
	}
}
