/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.optim;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.lib.BasicOperand;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.lib.Or;
import org.objectweb.medor.filter.lib.Not;
import org.objectweb.medor.filter.lib.UMinus;
import org.objectweb.medor.filter.lib.DivideBy;
import org.objectweb.medor.filter.lib.Greater;
import org.objectweb.medor.filter.lib.And;
import org.objectweb.medor.filter.lib.ExpressionPrinter;
import org.objectweb.medor.filter.TestExpressionHelper;
import org.objectweb.medor.optim.lib.PushNotInExpressionRule;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.query.TestQueryTreeHelper;
import org.objectweb.medor.lib.BasicField;
import org.objectweb.medor.lib.TestMedorHelper;
import org.objectweb.medor.type.lib.QTypeTuple;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.util.monolog.api.Logger;

/*
 * @author S.Chassande-Barrioz
 */

public class TestPushNotInExpressionRule extends TestCase {

	/**
	 * main method to launch the tests manually
	 */
	public static void main(String[] args) {
		TestRunner.run(suite());
	}

	/**
	 * This method creates a TestSuite object with the current tests
	 */
	public static Test suite() {
		return new TestSuite(TestPushNotInExpressionRule.class);
	}
	Logger logger = null;

	public TestPushNotInExpressionRule() {
		super("TestPushNotInExpressionRule");
		logger = new TestMedorHelper();
	}

	public TestPushNotInExpressionRule(String n) {
		super(n);
	}


	protected void setUp() {
	}

	protected void tearDown() {
	}

	public void testPushNotInExpression() {
		Expression e1, e2, e3, e4;
		e1 = new BasicOperand(true);
		e2 = new BasicFieldOperand(new BasicField("b", PTypeSpace.BOOLEAN));
		e1 = new Or(e1, e2);
		e1 = new Not(e1);
		e2 = new BasicFieldOperand(new BasicField("a", PTypeSpace.LONG));
		e2 = new UMinus(e2);
		e3 = new BasicOperand((long) 3);
		e2 = new DivideBy(e2, e3);
		e3 = new BasicOperand((long) 12);
		e2 = new Greater(e2, e3);
		e1 = new And(e2, e1);

		PushNotInExpressionRule pnier = new PushNotInExpressionRule();
		PushNotInExpressionRule.ModifiedExpression me = null;
		TestMedorHelper.debug("Before: exp=" + ExpressionPrinter.e2str(e1));
		try {
			me = pnier.pushNotInExpression(e1, false);
		}
		catch (MedorException e) {
			e.printStackTrace();
			fail("Impossible to pushNotInExpression" + e.getMessage());
		}
		TestMedorHelper.debug("After: exp=" + ExpressionPrinter.e2str(me.e));
		assertTrue("The expression has not been modified", me.isModified);

		e2 = new BasicFieldOperand(new BasicField("a", PTypeSpace.LONG));
		e2 = new UMinus(e2);
		e3 = new BasicOperand((long) 3);
		e2 = new DivideBy(e2, e3);
		e3 = new BasicOperand((long) 12);
		e1 = new Greater(e2, e3);

		e2 = new BasicOperand(true);
		e3 = new BasicFieldOperand(new BasicField("b", PTypeSpace.BOOLEAN));
		e2 = new And(new Not(e2), new Not(e3));
		e1 = new And(e1, e2);

		try {
			TestExpressionHelper.equals("", me.e, e1);
		}
		catch (MedorException e) {
			fail(e.getMessage());
		}
	}
}
