
/**
 *  MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.optim.jorm;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.medor.query.rdb.lib.BasicRdbExpQueryLeaf;
import org.objectweb.medor.query.rdb.lib.BasicQualifiedTable;
import org.objectweb.medor.query.rdb.api.QualifiedTable;
import org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.api.CalculatedField;
import org.objectweb.medor.query.lib.Project;
import org.objectweb.medor.query.lib.JoinProject;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.query.lib.QueryTreePrinter;
import org.objectweb.medor.query.jorm.lib.ClassExtent;
import org.objectweb.medor.query.TestQueryTreeHelper;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.filter.jorm.lib.CompositePName;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.filter.jorm.lib.EncodePName;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.lib.Equal;
import org.objectweb.medor.filter.lib.And;
import org.objectweb.medor.filter.lib.BasicParameterOperand;
import org.objectweb.medor.lib.TestMedorHelper;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.optim.api.QueryRewriter;
import org.objectweb.medor.optim.api.RewriteRule;
import org.objectweb.medor.optim.lib.DropUselessNodeRule;
import org.objectweb.medor.jorm.TestJormHelper;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.metainfo.api.CompositeName;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.Class;

import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;

public abstract class TestJormGoUpDecodeRule extends TestJormPDHelper {

	/**
	 * main method to launch the tests manually
	 */
	public static void main(String[] args) {
		TestRunner.run(suite());
	}

	/**
	 * This method creates a TestSuite object with the current tests
	 */
	public static Test suite() {
		return new TestSuite(TestJormGoUpDecodeRule.class);
	}

	public TestJormGoUpDecodeRule() {
		super("TestJormGoUpDecodeRule");
	}

	public TestJormGoUpDecodeRule(String n) {
		super(n);
	}

    public abstract JormLeafRewriter createJormLeafRewriter();

	public Example getExtent(Collection fileNames,
                             String className,
                             String pnameFieldName,
                             String[] fieldNames) {
		Example e = super.getExtent(fileNames, className, pnameFieldName, fieldNames);
		e.extent.setPMapper(mapper);
		return e;
	}

	Example eb;
	Example ec;
	Example ed;

	protected void setUp() {
		eb = getRewritedB();
		ec = getRewritedC();
		ed = getRewritedD();
	}

	protected void tearDown() {
	}

	public QueryTree getBCQueryTree(TestJormHelper tjh) {
        try {
            if (eb == null || ec == null)
                fail("All objects are not created by jorm"
                        + "\n-eb=" + eb
                        + "\n-ec=" + ec);
            QueryTreeField f1, f2;
			// Join B.y = C.x
			JoinProject j1 = new JoinProject("", null);
			Expression e = new Equal(
					new BasicFieldOperand(eb.qt.getTupleStructure().getField(eb.clazz.getFQName() + ".y")),
					new BasicFieldOperand(ec.qt.getTupleStructure().getField(ec.clazz.getFQName() + ".x"))
			);
			j1.setQueryFilter(e);
			f1 = (QueryTreeField) eb.qt.getTupleStructure().getField(eb.clazz.getFQName() + ".object");
			j1.addPropagatedField(f1.getName(), f1.getType(), new QueryTreeField[]{f1});
			f2 = (QueryTreeField) ec.qt.getTupleStructure().getField(ec.clazz.getFQName() + ".object");
			j1.addPropagatedField(f2.getName(), f2.getType(), new QueryTreeField[]{f2});
            return j1;
        } catch (MedorException e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
        return null;
	}

    public QueryTree getBCDQueryTree(TestJormHelper tjh) {
        try {
            if (eb == null || ec == null || ed == null)
                fail("All objects are not created by jorm"
                        + "\n-eb=" + eb
                        + "\n-ec=" + ec
                        + "\n-ed=" + ed);
            QueryTreeField f1, f2;

            // Join B.y = C.x
            JoinProject j1 = new JoinProject("", null);
            Expression e = new Equal(
                    new BasicFieldOperand(eb.qt.getTupleStructure().getField(eb.clazz.getFQName() + ".y")),
                    new BasicFieldOperand(ec.qt.getTupleStructure().getField(ec.clazz.getFQName() + ".x"))
            );
            j1.setQueryFilter(e);
            f1 = (QueryTreeField) eb.qt.getTupleStructure().getField(eb.clazz.getFQName() + ".object");
            j1.addPropagatedField(f1.getName(), f1.getType(), new QueryTreeField[]{f1});
            f2 = (QueryTreeField) ec.qt.getTupleStructure().getField(ec.clazz.getFQName() + ".object");
            j1.addPropagatedField(f2.getName(), f2.getType(), new QueryTreeField[]{f2});

            // Join B.object = C.object
            JoinProject j2 = new JoinProject("", null);
            f1 = (QueryTreeField) j1.getTupleStructure().getField(eb.clazz.getFQName() + ".object");
            f2 = (QueryTreeField) ed.qt.getTupleStructure().getField(ed.clazz.getFQName() + ".object");
            e = new Equal(new BasicFieldOperand(f1), new BasicFieldOperand(f2));
            j2.setQueryFilter(e);
            f1 = (QueryTreeField) ed.qt.getTupleStructure().getField(ed.clazz.getFQName() + ".c");
            f2 = (QueryTreeField) j1.getTupleStructure().getField(ec.clazz.getFQName() + ".object");
            j2.addPropagatedField(f1.getName(), f1.getType(), new QueryTreeField[]{f1});
            j2.addPropagatedField(f2.getName(), f2.getType(), new QueryTreeField[]{f2});

            if (TestMedorHelper.debug)
                QueryTreePrinter.printQueryTree(j2, logger);
            return j2;

        } catch (MedorException e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
        return null;
    }

	public void testPnameEqualParam1() {
		QueryTree j1 = getBCQueryTree(this);
		SelectProject qt = new SelectProject("", null);

		try {
			//filter
			QueryTreeField f1 = (QueryTreeField)
				j1.getTupleStructure().getField(eb.clazz.getFQName() + ".object");
			ParameterOperand po = new BasicParameterOperand(f1.getType(), "param1");
			qt.setQueryFilter(new Equal(new BasicFieldOperand(f1),po));
			//projection
			f1 = (QueryTreeField)
				j1.getTupleStructure().getField(ec.clazz.getFQName() + ".object");
			qt.addPropagatedField(f1.getName(), f1.getType(), new QueryTreeField[]{f1});
		} catch (MedorException e) {
			e.printStackTrace();
            fail(e.getMessage());
		}

		JormLeafRewriter jlr = createJormLeafRewriter();
        logger.log(BasicLevel.DEBUG, "JormLeafRewriter created");

		QueryTree rewrited = null;
        try {
            rewrited = new JormGoUpDecodeRule().rewrite(qt);
            rewrited = new DropUselessNodeRule().rewrite(rewrited);
        } catch (MedorException e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
        logger.log(BasicLevel.DEBUG, "BC1 rewrited");
        logger.log(BasicLevel.DEBUG, "=========================================");

        if (TestMedorHelper.debug)
            QueryTreePrinter.printQueryTree(rewrited, logger);

        logger.log(BasicLevel.DEBUG, "Checking of One");

		try {
			eb = getRewritedB();
			ec = getRewritedC();
			ed = getRewritedD();
            //build the expected tree
			QueryLeaf qlb = (QueryLeaf) ((QueryNode) eb.qt).getChildren()[0];
			QueryLeaf qlc = (QueryLeaf) ((QueryNode) ec.qt).getChildren()[0];
			//Node 1
			Expression e = new Equal(
                    new BasicFieldOperand(qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".y")),
                    new BasicFieldOperand(qlc.getTupleStructure().getField(ec.clazz.getFQName() + ".x")));
            QueryNode qn1 = new JoinProject(e, "", null);
            QueryTreeField qtf = null;
			qtf = (QueryTreeField) qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".c2");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});
            qtf = (QueryTreeField) qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".c1");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});
            qtf = (QueryTreeField) qlc.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});

			//node 2
			Field c1 = qn1.getTupleStructure().getField(eb.clazz.getFQName() + ".c1");
			Field c2 = qn1.getTupleStructure().getField(eb.clazz.getFQName() + ".c2");
			String paramName = "param1";
			e = new And(
				new Equal(
					new BasicFieldOperand(c2),
					new EncodePName(
						c2.getType(),
						paramName,
						"f2")
					),
				new Equal(
					new BasicFieldOperand(c1),
					new EncodePName(
						c1.getType(),
						paramName,
						"f1")
					)
			);
			QueryNode qn2 = new SelectProject(e, "", null);
			qtf = (QueryTreeField) qn1.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
			qn2.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});

			//Node 3
			QueryNode qn3 = new Project("", null);
            qtf = (QueryTreeField) qn2.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
            CalculatedField decodeField = (CalculatedField)
                    ec.qt.getTupleStructure().getField(ec.clazz.getFQName() + ".object");
            SinglePName spn = (SinglePName) decodeField.getExpression();
            qn3.addCalculatedField(
                    decodeField.getName(),
                    decodeField.getType(),
                    new SinglePName(
                            new BasicFieldOperand(qtf),
                            (ParameterOperand) spn.getRightExpression(),
                            spn.getType()));


			logger.log(BasicLevel.DEBUG, "BC1 expected");
			logger.log(BasicLevel.DEBUG, "=========================================");

			if (TestMedorHelper.debug)
				QueryTreePrinter.printQueryTree(qn3, logger);
			TestQueryTreeHelper.equals("", qn3, rewrited);
		} catch (MedorException me) {
            me.printStackTrace();
            fail(me.getMessage());
        }
	}

	public void testPnameEqualParam2() {
		QueryTree j1 = getBCQueryTree(this);
		SelectProject qt = new SelectProject("", null);

		try {
			//filter
			QueryTreeField f1 = (QueryTreeField)
				j1.getTupleStructure().getField(eb.clazz.getFQName() + ".object");
			ParameterOperand po = new BasicParameterOperand(f1.getType(), "param1");
			qt.setQueryFilter(new Equal(po, new BasicFieldOperand(f1)));
			//projection
			f1 = (QueryTreeField)
				j1.getTupleStructure().getField(ec.clazz.getFQName() + ".object");
			qt.addPropagatedField(f1.getName(), f1.getType(), new QueryTreeField[]{f1});
		} catch (MedorException e) {
			e.printStackTrace();
            fail(e.getMessage());
		}

		JormLeafRewriter jlr = createJormLeafRewriter();
        logger.log(BasicLevel.DEBUG, "JormLeafRewriter created");

		QueryTree rewrited = null;
        try {
            rewrited = new JormGoUpDecodeRule().rewrite(qt);
            rewrited = new DropUselessNodeRule().rewrite(rewrited);
        } catch (MedorException e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
        logger.log(BasicLevel.DEBUG, "BC2 rewrited");
        logger.log(BasicLevel.DEBUG, "=========================================");

        if (TestMedorHelper.debug)
            QueryTreePrinter.printQueryTree(rewrited, logger);

        logger.log(BasicLevel.DEBUG, "Checking of One");

		try {
			eb = getRewritedB();
			ec = getRewritedC();
			ed = getRewritedD();
            //build the expected tree
			QueryLeaf qlb = (QueryLeaf) ((QueryNode) eb.qt).getChildren()[0];
			QueryLeaf qlc = (QueryLeaf) ((QueryNode) ec.qt).getChildren()[0];
			//Node 1
			Expression e = new Equal(
                    new BasicFieldOperand(qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".y")),
                    new BasicFieldOperand(qlc.getTupleStructure().getField(ec.clazz.getFQName() + ".x")));
            QueryNode qn1 = new JoinProject(e, "", null);
            QueryTreeField qtf = null;
			qtf = (QueryTreeField) qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".c2");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});
            qtf = (QueryTreeField) qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".c1");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});
            qtf = (QueryTreeField) qlc.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});

			//node 2
			Field c1 = qn1.getTupleStructure().getField(eb.clazz.getFQName() + ".c1");
			Field c2 = qn1.getTupleStructure().getField(eb.clazz.getFQName() + ".c2");
			String paramName = "param1";
			e = new And(
				new Equal(
					new EncodePName(
						c2.getType(),
						paramName,
						"f2"),
					new BasicFieldOperand(c2)
					),
				new Equal(
					new EncodePName(
						c1.getType(),
						paramName,
						"f1"),
					new BasicFieldOperand(c1)
					)
			);
			QueryNode qn2 = new SelectProject(e, "", null);
			qtf = (QueryTreeField) qn1.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
			qn2.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});

			//Node 3
			QueryNode qn3 = new Project("", null);
            qtf = (QueryTreeField) qn2.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
            CalculatedField decodeField = (CalculatedField)
                    ec.qt.getTupleStructure().getField(ec.clazz.getFQName() + ".object");
            SinglePName spn = (SinglePName) decodeField.getExpression();
            qn3.addCalculatedField(
                    decodeField.getName(),
                    decodeField.getType(),
                    new SinglePName(
                            new BasicFieldOperand(qtf),
                            (ParameterOperand) spn.getRightExpression(),
                            spn.getType()));


			logger.log(BasicLevel.DEBUG, "BC2 expected");
			logger.log(BasicLevel.DEBUG, "=========================================");

			if (TestMedorHelper.debug)
				QueryTreePrinter.printQueryTree(qn3, logger);
			TestQueryTreeHelper.equals("", qn3, rewrited);
		} catch (MedorException me) {
            me.printStackTrace();
            fail(me.getMessage());
        }
	}

	public void testPnameEqualParam3() {
		QueryTree j1 = getBCQueryTree(this);
		SelectProject qt = new SelectProject("", null);

		try {
			//filter
			QueryTreeField f1 = (QueryTreeField)
				j1.getTupleStructure().getField(ec.clazz.getFQName() + ".object");
			ParameterOperand po = new BasicParameterOperand(f1.getType(), "param1");
			qt.setQueryFilter(new Equal(po, new BasicFieldOperand(f1)));
			//projection
			f1 = (QueryTreeField)
				j1.getTupleStructure().getField(ec.clazz.getFQName() + ".object");
			qt.addPropagatedField(f1.getName(), f1.getType(), new QueryTreeField[]{f1});
		} catch (MedorException e) {
			e.printStackTrace();
            fail(e.getMessage());
		}

		JormLeafRewriter jlr = createJormLeafRewriter();
        logger.log(BasicLevel.DEBUG, "JormLeafRewriter created");

		QueryTree rewrited = null;
        try {
            rewrited = new JormGoUpDecodeRule().rewrite(qt);
            rewrited = new DropUselessNodeRule().rewrite(rewrited);
        } catch (MedorException e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
        logger.log(BasicLevel.DEBUG, "BC3 rewrited");
        logger.log(BasicLevel.DEBUG, "=========================================");

        if (TestMedorHelper.debug)
            QueryTreePrinter.printQueryTree(rewrited, logger);

        logger.log(BasicLevel.DEBUG, "Checking of One");

		try {
			eb = getRewritedB();
			ec = getRewritedC();
			ed = getRewritedD();
            //build the expected tree
			QueryLeaf qlb = (QueryLeaf) ((QueryNode) eb.qt).getChildren()[0];
			QueryLeaf qlc = (QueryLeaf) ((QueryNode) ec.qt).getChildren()[0];
			//Node 1
			Expression e = new Equal(
                    new BasicFieldOperand(qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".y")),
                    new BasicFieldOperand(qlc.getTupleStructure().getField(ec.clazz.getFQName() + ".x")));
            QueryNode qn1 = new JoinProject(e, "", null);
            QueryTreeField qtf = null;
			qtf = (QueryTreeField) qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".c2");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});
            qtf = (QueryTreeField) qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".c1");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});
            qtf = (QueryTreeField) qlc.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});

			//node 2
			Field id = qn1.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
			String paramName = "param1";
			e = new Equal(
				new EncodePName(id.getType(), paramName),
				new BasicFieldOperand(id));
			QueryNode qn2 = new SelectProject(e, "", null);
			qtf = (QueryTreeField) qn1.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
			qn2.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});

			//Node 3
			QueryNode qn3 = new Project("", null);
            qtf = (QueryTreeField) qn2.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
            CalculatedField decodeField = (CalculatedField)
                    ec.qt.getTupleStructure().getField(ec.clazz.getFQName() + ".object");
            SinglePName spn = (SinglePName) decodeField.getExpression();
            qn3.addCalculatedField(
                    decodeField.getName(),
                    decodeField.getType(),
                    new SinglePName(
                            new BasicFieldOperand(qtf),
                            (ParameterOperand) spn.getRightExpression(),
                            spn.getType()));


			logger.log(BasicLevel.DEBUG, "BC3 expected");
			logger.log(BasicLevel.DEBUG, "=========================================");

			if (TestMedorHelper.debug)
				QueryTreePrinter.printQueryTree(qn3, logger);
			TestQueryTreeHelper.equals("", qn3, rewrited);
		} catch (MedorException me) {
            me.printStackTrace();
            fail(me.getMessage());
        }
	}

    public void testBCD() {
        QueryTree qt = getBCDQueryTree(this);
        JormLeafRewriter jlr = createJormLeafRewriter();
        logger.log(BasicLevel.DEBUG, "JormLeafRewriter created");

        QueryTree rewrited = null;
        try {
            rewrited = new JormGoUpDecodeRule().rewrite(qt);
            rewrited = new DropUselessNodeRule().rewrite(rewrited);
        } catch (MedorException e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
        logger.log(BasicLevel.DEBUG, "BCD rewrited");
        logger.log(BasicLevel.DEBUG, "=========================================");

        if (TestMedorHelper.debug)
            QueryTreePrinter.printQueryTree(rewrited, logger);

        logger.log(BasicLevel.DEBUG, "Checking of One");

        try {
            //build the expected tree
            Example eb = getRewritedB();
            QueryLeaf qlb = (QueryLeaf) ((QueryNode) eb.qt).getChildren()[0];

            Example ec = getRewritedC();
            QueryLeaf qlc = (QueryLeaf) ((QueryNode) ec.qt).getChildren()[0];

            Example ed = getRewritedD();
            QueryLeaf qld = (QueryLeaf) ((QueryNode) ed.qt).getChildren()[0];

            Expression e = new Equal(
                    new BasicFieldOperand(qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".y")),
                    new BasicFieldOperand(qlc.getTupleStructure().getField(ec.clazz.getFQName() + ".x")));
            QueryNode qn1 = new JoinProject(e, "", null);
            QueryTreeField qtf = null;
			qtf = (QueryTreeField) qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".c2");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});
            qtf = (QueryTreeField) qlb.getTupleStructure().getField(eb.clazz.getFQName() + ".c1");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});
            qtf = (QueryTreeField) qlc.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
            qn1.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});

            e = new And(
                    new Equal(
                            new BasicFieldOperand(qn1.getTupleStructure().getField(eb.clazz.getFQName() + ".c2")),
                            new BasicFieldOperand(qld.getTupleStructure().getField(ed.clazz.getFQName() + ".b"))),
                    new Equal(
                            new BasicFieldOperand(qn1.getTupleStructure().getField(eb.clazz.getFQName() + ".c1")),
                            new BasicFieldOperand(qld.getTupleStructure().getField(ed.clazz.getFQName() + ".a"))
                    )
            );
            QueryNode qn2 = new JoinProject(e, "", null);
            qtf = (QueryTreeField) qld.getTupleStructure().getField(ed.clazz.getFQName() + ".c");
            qn2.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});
            qtf = (QueryTreeField) qn1.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
            qn2.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});

            QueryNode qn3 = new Project("", null);
            qtf = (QueryTreeField) qn2.getTupleStructure().getField(ec.clazz.getFQName() + ".id");
            CalculatedField decodeField = (CalculatedField)
                    ec.qt.getTupleStructure().getField(ec.clazz.getFQName() + ".object");
            SinglePName spn = (SinglePName) decodeField.getExpression();
            qn3.addCalculatedField(
                    decodeField.getName(),
                    decodeField.getType(),
                    new SinglePName(
                            new BasicFieldOperand(qtf),
                            (ParameterOperand) spn.getRightExpression(),
                            spn.getType()));
            qtf = (QueryTreeField) qn2.getTupleStructure().getField(ed.clazz.getFQName() + ".c");
            qn3.addPropagatedField(qtf.getName(), qtf.getType(), new QueryTreeField[]{qtf});

            TestQueryTreeHelper.equals("", qn3, rewrited);
        } catch (MedorException me) {
            me.printStackTrace();
            fail(me.getMessage());
        }
    }
}
