/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.optim.jorm;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.lib.BasicParameterOperand;
import org.objectweb.medor.query.TestQueryTreeHelper;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.Project;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;

import java.util.Collection;

/**
 * @author S.Chassande-Barrioz
 */
public abstract class TestJormLeafRewriter extends TestJormPDHelper {

    public TestJormLeafRewriter() {
        super("TestJormLeafRewriter");
    }

    public TestJormLeafRewriter(String n) {
        super(n);
    }

    protected abstract QueryLeaf getQueryLeaf_A() throws MedorException;

    protected abstract QueryLeaf getQueryLeaf_One() throws MedorException;


    public Example getExtent(Collection fileNames,
                             String className,
                             String pnameFieldName,
                             String[] fieldNames) {
		Example e = super.getExtent(fileNames, className, pnameFieldName, fieldNames);
		e.extent.setPMapper(mapper);
		return e;
	}

    public void testA() {
        checkRewritedA(getRewritedA());
    }

    public void testOne() {
        checkRewritedOne(getRewritedOne());
    }

    public void testB() {
        checkRewritedB(getRewritedB());
    }

    public void testC() {
        checkRewritedC(getRewritedC());
    }

    public void testD() {
        checkRewritedD(getRewritedD());
    }

    public void checkRewritedA(Example e) {
        try {
            String prefix = e.clazz.getFQName() + ".";
            QueryLeaf ql = getQueryLeaf_A();
            QueryTreeField id = (QueryTreeField)
                ql.getTupleStructure().getField(prefix + "id");
            QueryTreeField f1 = (QueryTreeField)
                ql.getTupleStructure().getField(prefix + "field1");
            QueryTreeField f2 = (QueryTreeField)
                ql.getTupleStructure().getField(prefix + "field2");

            Project expected = new Project("", null);
            SinglePName spn = new SinglePName(
                new BasicFieldOperand(id),
                new BasicParameterOperand(
                    PTypeSpaceMedor.NAMING_CONTEXT, prefix + "object"),
                e.extent.getField(prefix + "object").getType());
            expected.addCalculatedField(prefix + "object", spn.getType(), spn);
            expected.addPropagatedField(
                prefix + "field1", f1.getType(), new QueryTreeField[]{f1});
            expected.addPropagatedField(
                prefix + "field2", f2.getType(), new QueryTreeField[]{f2});
            TestQueryTreeHelper.equals("", expected, e.qt);
        }
        catch (MedorException me) {
            me.printStackTrace();
            fail(me.getMessage());
        }
    }

    public void checkRewritedOne(Example e) {
        try {
            QueryLeaf ql = getQueryLeaf_One();
            String prefix = e.clazz.getFQName() + ".";
            QueryTreeField id = (QueryTreeField)
                ql.getTupleStructure().getField(prefix + "id");
            QueryTreeField f1 = (QueryTreeField)
                ql.getTupleStructure().getField(prefix + "field1");
            QueryTreeField ida = (QueryTreeField)
                ql.getTupleStructure().getField(prefix + "idA");

            Project expected = new Project("", null);
            SinglePName spn = new SinglePName(
                new BasicFieldOperand(id),
                new BasicParameterOperand(
                    PTypeSpaceMedor.NAMING_CONTEXT, prefix + "." + "object"),
                e.extent.getField(prefix + "object").getType());
            expected.addCalculatedField(prefix + "object", spn.getType(), spn);
            expected.addPropagatedField(
                prefix + "field1", f1.getType(), new QueryTreeField[]{f1});
            String poName = prefix + "fieldA";
            spn = new SinglePName(
                new BasicFieldOperand(id),
                new BasicParameterOperand(
                    PTypeSpaceMedor.NAMING_CONTEXT, poName),
                e.extent.getField(prefix + "fieldA").getType());
            expected.addCalculatedField(prefix + "fieldA", spn.getType(), spn);
            TestQueryTreeHelper.equals("", expected, e.qt);
        }
        catch (MedorException me) {
            me.printStackTrace();
            fail(me.getMessage());
        }
    }

    public void checkRewritedB(Example e) {
//TODO
    }

    public void checkRewritedC(Example e) {
//TODO
    }

    public void checkRewritedD(Example e) {
//TODO
    }
}
