/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.optim.jorm;

import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.lib.TestMedorHelper;
import org.objectweb.medor.query.TestQueryTreeHelper;
import org.objectweb.medor.query.lib.Project;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.jorm.lib.ClassExtent;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.lib.BasicParameterOperand;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.jorm.TestJormHelper;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * @author S.Chassande-Barrioz
 */
public abstract class TestJormPDHelper extends TestJormHelper {

    public TestJormPDHelper() {
        super("TestJormPDHelper");
    }

    public TestJormPDHelper(String n) {
        super(n);
    }

    public Example getExtentOfA() {
        ArrayList al = new ArrayList();
        al.add("org/objectweb/medor/optim/jorm/rdb/A.pd");
        return getExtent(al, "A");
    }

    public Example getExtentOfOne() {
        ArrayList al = new ArrayList();
        al.add("org/objectweb/medor/optim/jorm/rdb/One.pd");
        return getExtent(al, "One");
    }

    public Example getExtentOfB() {
        ArrayList al = new ArrayList();
        al.add("org/objectweb/medor/optim/jorm/rdb/ByteCharCN.pd");
        al.add("org/objectweb/medor/optim/jorm/rdb/B.pd");
        return getExtent(al, "B");
    }

    public Example getExtentOfC() {
        ArrayList al = new ArrayList();
        al.add("org/objectweb/medor/optim/jorm/rdb/C.pd");
        return getExtent(al, "C");
    }

    public Example getExtentOfD() {
        ArrayList al = new ArrayList();
        al.add("org/objectweb/medor/optim/jorm/rdb/ByteCharCN.pd");
        al.add("org/objectweb/medor/optim/jorm/rdb/D.pd");
        return getExtent(al, "D");
    }

    public Example getRewritedA() {
        return getRewrited(getExtentOfA());
    }

    public Example getRewritedOne() {
        return getRewrited(getExtentOfOne());
    }

    public Example getRewritedB() {
        return getRewrited(getExtentOfB());
    }

    public Example getRewritedC() {
        return getRewrited(getExtentOfC());
    }

    public Example getRewritedD() {
        return getRewrited(getExtentOfD());
    }
}
