/**
 * copyright area
 */
package org.objectweb.medor.optim.jorm;

import junit.framework.Test;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryNode;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.query.lib.Project;
import org.objectweb.medor.query.lib.QueryTreePrinter;
import org.objectweb.medor.query.TestQueryTreeHelper;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.lib.Equal;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.lib.BasicParameterOperand;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.jorm.type.api.PTypeSpace;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Collection;

/**
 * @author S.Chassande-Barrioz
 */
public abstract class TestJormQueryRewriter extends TestJormPDHelper {

    public TestJormQueryRewriter() {
		super("TestJormQueryRewriter");
	}

	public TestJormQueryRewriter(String n) {
		super(n);
	}

    public abstract QueryLeaf getQueryLeafA(Example e) throws MedorException;

    public void test1() {
        try {
            Example e = getExtentOfA();
            Expression exp = new Equal(
                new BasicFieldOperand(e.extent.getField(e.clazz.getFQName() + ".field1")),
                new BasicParameterOperand(PTypeSpace.STRING, "?1"));
            QueryNode qn = new SelectProject(exp, "", null);
            QueryTreeField qtf = (QueryTreeField)
                e.extent.getField(e.extent.getPNameFieldName());
            qn.addPropagatedField(
                qtf.getName(),
                qtf.getType(),
                new QueryTreeField[]{qtf});
            Map name2Mapper = new HashMap(1);
            name2Mapper.put(e.extent.getName(), mapper);
            JormQueryRewriter jqr = new JormQueryRewriter(
                name2Mapper,
                Collections.singleton(createJormLeafRewriter()));
            QueryTree result = jqr.transform(qn);
            QueryTreePrinter.printQueryTree(result, logger);
            QueryLeaf leaf = getQueryLeafA(e);
            QueryNode expected = new Project("", null);
            expected.addCalculatedField(
                e.clazz.getFQName() + ".object",
                e.extent.getField(e.extent.getPNameFieldName()).getType(),
                new SinglePName(
                    new BasicFieldOperand(
                        leaf.getTupleStructure().getField(e.clazz.getFQName() + ".id")),
                    new BasicParameterOperand(
                        PTypeSpaceMedor.NAMING_CONTEXT,
                        e.clazz.getName()),
                    e.extent.getField(e.clazz.getFQName() + ".object").getType()
                )
            );
            QueryTreePrinter.printQueryTree(expected, logger);
            TestQueryTreeHelper.equals("", expected, result);
        }
        catch (MedorException e1) {
            e1.printStackTrace();
            fail(e1.getMessage());
        }
    }
}
