/**
 * copyright area
 */
package org.objectweb.medor.optim.jorm.rdb;

import org.objectweb.medor.optim.jorm.TestJormQueryRewriter;
import org.objectweb.medor.optim.jorm.JormLeafRewriter;
import org.objectweb.medor.optim.jorm.RdbJormLeafRewriter;
import org.objectweb.medor.lib.TestMedorHelper;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf;
import org.objectweb.medor.query.rdb.api.QualifiedTable;
import org.objectweb.medor.query.rdb.lib.BasicRdbExpQueryLeaf;
import org.objectweb.medor.query.rdb.lib.BasicQualifiedTable;
import org.objectweb.medor.filter.lib.Equal;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.lib.BasicParameterOperand;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.api.MedorException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.runtime.MapperJDBC;
import org.objectweb.jorm.runtime.ConnectionSpecJDBC;
import org.objectweb.jorm.metainfo.api.rdb.RDBClassMapping;
import org.objectweb.jorm.type.api.PTypeSpace;
import junit.textui.TestRunner;
import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * @author S.Chassande-Barrioz
 */
public class TestRdbJormQueryRewriter extends TestJormQueryRewriter {
    private static String mapperName;
	private static String mappingName;
	private static String miMappingFactory;
	private static String mappingParser;
	private static String pdPath;
	private static String dtdPath;

	static {
		mapperName = System.getProperty(
			"medor.jorm.mapperName", "rdb");
		TestMedorHelper.debug("MapperName = " + mapperName);

		mappingName = System.getProperty(
			"medor.jorm.mappingName", "");
		TestMedorHelper.debug("MappingName = " + mappingName);

		miMappingFactory = System.getProperty(
			"medor.jorm.miMappingName",
			"org.objectweb.jorm.metainfo.lib.rdb.BasicRDBMappingFactory");
		TestMedorHelper.debug("MIMappingFactory = " + miMappingFactory);

		mappingParser = System.getProperty(
			"medor.jorm.miMappingName",
			"org.objectweb.jorm.parser.lib.rdb.RDBParser");
		TestMedorHelper.debug("MappingParser = " + mappingParser);

		pdPath = System.getProperty(
			"medor.jorm.pd.path", "./");
		TestMedorHelper.debug("pd Path = " + pdPath);
		dtdPath = System.getProperty(
			"medor.jorm.dtd.path", "./");
		TestMedorHelper.debug("dtd Path = " + dtdPath);
	}

	/**
	 * main method to launch the tests manually
	 */
	public static void main(String[] args) {
		TestRunner.run(suite());
	}

	/**
	 * This method creates a TestSuite object with the current tests
	 */
	public static Test suite() {
		return new TestSuite(TestRdbJormQueryRewriter.class);
	}

	public TestRdbJormQueryRewriter() {
		super("TestRdbJormQueryRewriter");
	}

	public TestRdbJormQueryRewriter(String n) {
		super(n);
	}

    public String getPDPath() {
        return pdPath;
    }

    public String getDTDPath() {
        return dtdPath;
    }

    public String getMapperName() {
		return mapperName;
	}

	public String getMappingName() {
		return mappingName;
	}

    public PMapper newMappper() {
        PMapper m = new MapperJDBC();
        try {
            ConnectionSpecJDBC cs = new ConnectionSpecJDBC("");
            m.setConnectionFactory(cs);
        }
        catch (PException e) {
            e.printStackTrace();
        }
        return m;
    }

	public String getMIMappingFactory() {
		return miMappingFactory;
	}

	public String getMappingParser() {
		return mappingParser;
	}

	public JormLeafRewriter createJormLeafRewriter() {
		return new RdbJormLeafRewriter();
	}

    public QueryLeaf getQueryLeafA(Example e) throws MedorException {
        RDBClassMapping cm = (RDBClassMapping)
            e.clazz.getClassMapping(getMapperName(), getMappingName());
        String name = e.clazz.getFQName();
        String prefix = name + ".";

        QualifiedTable qt = new BasicQualifiedTable(cm.getTableName(), null);
        BasicRdbExpQueryLeaf leaf = new BasicRdbExpQueryLeaf(
            null, new QualifiedTable[]{qt}, "");
        leaf.addRdbField(prefix + "id", PTypeSpace.LONG, "id_col", qt);
        Field f = leaf.addRdbField(prefix + "field1", PTypeSpace.STRING, "FIELD_1", qt);
        leaf.addRdbField(prefix + "field2", PTypeSpace.STRING, "FIELD_2", qt);
        leaf.setQueryFilter(new Equal(
            new BasicFieldOperand(f),
            new BasicParameterOperand(PTypeSpace.STRING, "?1")));
        return leaf;
    }
}
