/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.query.jorm.rdb;

import org.objectweb.medor.query.jorm.TestNavigatorNodeFactory;
import org.objectweb.medor.optim.jorm.JormLeafRewriter;
import org.objectweb.medor.optim.jorm.RdbJormLeafRewriter;
import org.objectweb.medor.optim.jorm.rdb.TestRdbJormGoUpDecodeRule;
import org.objectweb.medor.lib.TestMedorHelper;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.runtime.MapperJDBC;
import org.objectweb.jorm.runtime.ConnectionSpecJDBC;
import junit.textui.TestRunner;
import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * @author S.Chassande-Barrioz
 */
public class TestRdbNavigatorNodeFactory extends TestNavigatorNodeFactory {
    private static String mapperName;
	private static String mappingName;
	private static String miMappingFactory;
	private static String mappingParser;
    private static String pdPath;
	private static String dtdPath;

	static {
		mapperName = System.getProperty(
			"medor.jorm.mapperName", "rdb");
		TestMedorHelper.debug("MapperName = " + mapperName);

		mappingName = System.getProperty(
			"medor.jorm.mappingName", "");
		TestMedorHelper.debug("MappingName = " + mappingName);

		miMappingFactory = System.getProperty(
			"medor.jorm.miMappingName",
			"org.objectweb.jorm.metainfo.lib.rdb.BasicRDBMappingFactory");
		TestMedorHelper.debug("MIMappingFactory = " + miMappingFactory);

		mappingParser = System.getProperty(
			"medor.jorm.miMappingName",
			"org.objectweb.jorm.parser.lib.rdb.RDBParser");
		TestMedorHelper.debug("MappingParser = " + mappingParser);

        pdPath = System.getProperty(
			"medor.jorm.pd.path", "./");
		TestMedorHelper.debug("pd Path = " + pdPath);
		dtdPath = System.getProperty(
			"medor.jorm.dtd.path", "./");
		TestMedorHelper.debug("dtd Path = " + dtdPath);
	}

	/**
	 * main method to launch the tests manually
	 */
	public static void main(String[] args) {
		TestRunner.run(suite());
	}

	/**
	 * This method creates a TestSuite object with the current tests
	 */
	public static Test suite() {
		return new TestSuite(TestRdbNavigatorNodeFactory.class);
	}

	public TestRdbNavigatorNodeFactory() {
		super("TestRdbNavigatorNodeFactory");
	}

	public TestRdbNavigatorNodeFactory(String n) {
		super(n);
	}

    public String getPDPath() {
        return pdPath;
    }

    public String getDTDPath() {
        return dtdPath;
    }

    public String getMapperName() {
		return mapperName;
	}

	public String getMappingName() {
		return mappingName;
	}

    public PMapper newMappper() {
        PMapper m = new MapperJDBC();
        try {
            ConnectionSpecJDBC cs =
                new ConnectionSpecJDBC("jdbc:product:basename", "toto", "pass");
            m.setConnectionFactory(cs);
        }
        catch (PException e) {
            e.printStackTrace();
        }
        return m;
    }

	public String getMIMappingFactory() {
		return miMappingFactory;
	}

	public String getMappingParser() {
		return mappingParser;
	}

	public JormLeafRewriter createJormLeafRewriter() {
		return new RdbJormLeafRewriter();
	}
}
