/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.query.rdb.lib;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.filter.api.Expression;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.api.ParameterOperand;
import org.objectweb.medor.filter.lib.And;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.filter.lib.BasicOperand;
import org.objectweb.medor.filter.lib.BasicParameterOperand;
import org.objectweb.medor.filter.lib.CollectionOperand;
import org.objectweb.medor.filter.lib.Greater;
import org.objectweb.medor.filter.lib.InCollection;
import org.objectweb.medor.filter.lib.Lower;
import org.objectweb.medor.lib.TestMedorHelper;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.rdb.api.QualifiedTable;
import org.objectweb.medor.query.rdb.api.RdbExpField;
import org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.util.monolog.api.Logger;

import java.util.ArrayList;
import java.util.Collection;

public class TestRewrite2SQL extends TestCase {

	static boolean debug = false;

	Logger logger = null;

	/**
	 * main method to launch the tests manually
	 */
	public static void main(String[] args) {
		TestRunner.run(suite());
	}

	/**
	 * This method creates a TestSuite object with the current tests
	 */
	public static Test suite() {
		return new TestSuite(TestRewrite2SQL.class);
	}

	public TestRewrite2SQL() {
		super("TestRewrite2SQL");
		logger = new TestMedorHelper();
	}

	public TestRewrite2SQL(String n) {
		super(n);
		debug = "true".equalsIgnoreCase(
			System.getProperty("medor.test.debug", "false"));
	}

	protected void setUp() {
	}

	protected void tearDown() {
	}

	public void testAliasesSimple() {
		QueryTree myRequest = null;
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable t2 = new BasicQualifiedTable("t2", "al2");
			QualifiedTable t3 = new BasicQualifiedTable("t3", "al3");
			QualifiedTable[] tables = {t1, t2};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			RdbExpField f2 = ql.addRdbField("f2", null, "col2", t2);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			FieldOperand fieldOp2 = new BasicFieldOperand(f2);
			BasicOperand valueL = new BasicOperand(1013);
			BasicOperand valueR = new BasicOperand("coucou");
			Expression left = new Lower(fieldOp1, valueL);
			Expression right = new Greater(fieldOp2, valueR);
			Expression filter = new And(left, right);
			//assigne the filter to the query leaf
			ql.setQueryFilter(filter);
			// Add an OrderBy Field
			ql.addOrderByField(f1);
			TestMedorHelper.debug("First request");
			//computes the SQL query
			TestMedorHelper.debug(ql.getSqlRequest());
			assertEquals("OK", ql.getSqlRequest(),
			"SELECT t1.col1, al2.col2 FROM t1, t2 as al2 WHERE ((t1.col1 < '1013') AND (al2.col2 > 'coucou')) ORDER BY t1.col1");
		}
		catch (Exception e) {
			fail(e.getMessage());
		}
	}
	public void testAliasesWithParam() {
		QueryTree myRequest = null;
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable t2 = new BasicQualifiedTable("t2", "al2");
			QualifiedTable t3 = new BasicQualifiedTable("t3", "al3");
			QualifiedTable[] tables = {t1, t2};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			RdbExpField f2 = ql.addRdbField("f2", null, "col2", t2);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			FieldOperand fieldOp2 = new BasicFieldOperand(f2);
			BasicOperand valueL = new BasicOperand(1013);
			BasicOperand valueR = new BasicOperand("coucou");
			Expression left = new Lower(fieldOp1, valueL);
			Expression right = new Greater(fieldOp2, valueR);
			Expression filter = new And(left, right);
			//assigne the filter to the query leaf
			ql.setQueryFilter(filter);
			// Add an OrderBy Field
			ql.addOrderByField(f1);
			ParameterOperand fieldOp3 =
				new BasicParameterOperand(PTypeSpaceMedor.STRING, "x");
			Expression left2 = new Lower(fieldOp3, valueL);
			filter = new And(left2, right);
			ql.setQueryFilter(filter);
			ql.setGroupBy(f2);
			TestMedorHelper.debug(ql.getSqlRequest());
			assertEquals("OK", ql.getSqlRequest(),
			"SELECT t1.col1, al2.col2 FROM t1, t2 as al2 WHERE ((? < '1013') AND (al2.col2 > 'coucou')) ORDER BY t1.col1 GROUP BY al2.col2");
		}
		catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testEmptyCollection() {
		QueryTree myRequest = null;
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable[] tables = {t1};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			Collection col = new ArrayList();
			CollectionOperand colOp = new CollectionOperand(col);
			Expression filter = new InCollection(fieldOp1, colOp, PTypeSpace.STRING);
			;
			//assigne the filter to the query leaf
			ql.setQueryFilter(filter);
			// Add an OrderBy Field
			ql.addOrderByField(f1);
			//computes the SQL query
			TestMedorHelper.debug(ql.getSqlRequest());
			fail("Empty collection not detected");
		}
		catch (Exception e) {
			assertEquals("detected", e.getMessage(), "Empty collection for InCollection operator");
			TestMedorHelper.debug("Empty collection detected");
			}
	}

	public void testCollOfStrings() {
		QueryTree myRequest = null;
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable[] tables = {t1};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			Collection col = new ArrayList();
			col.add("value1");
			col.add("val2");
			CollectionOperand colOp = new CollectionOperand(col);
			Expression filter = new InCollection(fieldOp1, colOp, PTypeSpace.STRING);

			//assigns the filter to the query leaf
			ql.setQueryFilter(filter);
			// Add an OrderBy Field
			ql.addOrderByField(f1);
			TestMedorHelper.debug("Collection of Strings");
			//computes the SQL query
			TestMedorHelper.debug(ql.getSqlRequest());
			assertEquals("OK", ql.getSqlRequest(),
			"SELECT t1.col1 FROM t1 WHERE (t1.col1 IN (\"value1\", \"val2\")) ORDER BY t1.col1");
		}
		catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testCollOfFloats() {
		QueryTree myRequest = null;
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable[] tables = {t1};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			Collection col = new ArrayList();
			col.add(new Float(123.34));
			col.add(new Float(345.23));
			CollectionOperand colOp = new CollectionOperand(col);
			Expression filter = new InCollection(fieldOp1, colOp, PTypeSpace.FLOAT);

			//assigns the filter to the query leaf
			ql.setQueryFilter(filter);
			// Add an OrderBy Field
			ql.addOrderByField(f1);
			TestMedorHelper.debug("Collection of floats");
			//computes the SQL query
			TestMedorHelper.debug(ql.getSqlRequest());
			assertEquals("OK", ql.getSqlRequest(),
			"SELECT t1.col1 FROM t1 WHERE (t1.col1 IN (123.34, 345.23)) ORDER BY t1.col1");
		}
		catch (Exception e) {
			fail(e.getMessage());
		}
	}
}
